@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2017 openpyxl
+# Copyright (c) 2010-2018 openpyxl
 
 import pytest
 
@@ -22,7 +22,9 @@ class TestRichText:
         <rich xmlns:a="http://schemas.openxmlformats.org/drawingml/2006/main">
           <a:bodyPr />
           <a:p>
-             <a:r />
+             <a:r>
+               <a:t />
+             </a:r>
           </a:p>
         </rich>
         """
