@@ -1,5 +1,5 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2015 openpyxl
+# Copyright (c) 2010-2018 openpyxl
 
 from openpyxl.compat import unicode
 
@@ -94,9 +94,9 @@ class Font(Serialisable):
     namespace = DRAWING_NS
 
     typeface = String()
-    panose = Typed(expected_type=HexBinary, allow_none=True)
-    pitchFamily = Typed(expected_type=MinMax, allow_none=True)
-    charset = Typed(expected_type=MinMax, allow_none=True)
+    panose = HexBinary(allow_none=True)
+    pitchFamily = MinMax(min=0, max=52, allow_none=True)
+    charset = Integer(allow_none=True)
 
     def __init__(self,
                  typeface=None,
@@ -118,7 +118,7 @@ class CharacterProperties(Serialisable):
     kumimoji = Bool(allow_none=True)
     lang = String(allow_none=True)
     altLang = String(allow_none=True)
-    sz = Integer(allow_none=True)
+    sz = MinMax(allow_none=True, min=100, max=400000) # 100ths of a point
     b = Bool(allow_none=True)
     i = Bool(allow_none=True)
     u = NoneSet(values=(['words', 'sng', 'dbl', 'heavy', 'dotted',
@@ -164,10 +164,10 @@ class CharacterProperties(Serialisable):
     uFillTx = EmptyTag()
     uFill = EmptyTag()
 
-    __elements__ = ('ln', 'highlight', 'latin', 'ea', 'cs', 'sym',
-                    'hlinkClick', 'hlinkMouseOver', 'rtl', 'noFill', 'solidFill', 'gradFill',
-                    'blipFill', 'pattFill', 'grpFill', 'effectLst', 'effectDag', 'uLnTx',
-                    'uLn', 'uFillTx', 'uFill')
+    __elements__ = ('ln', 'noFill', 'solidFill', 'gradFill', 'blipFill',
+                    'pattFill', 'grpFill', 'effectLst', 'effectDag', 'highlight','uLnTx',
+                    'uLn', 'uFillTx', 'uFill', 'latin', 'ea', 'cs', 'sym', 'hlinkClick',
+                    'hlinkMouseOver', 'rtl', )
 
     def __init__(self,
                  kumimoji=None,
@@ -279,8 +279,8 @@ class TabStopList(Serialisable):
 
 class Spacing(Serialisable):
 
-    spcPct = NestedInteger()
-    spcPts = NestedInteger()
+    spcPct = NestedInteger(allow_none=True)
+    spcPts = NestedInteger(allow_none=True)
 
     __elements__ = ('spcPct', 'spcPts')
 
@@ -471,14 +471,14 @@ class RegularTextRun(Serialisable):
 
     rPr = Typed(expected_type=CharacterProperties, allow_none=True)
     properties = Alias("rPr")
-    t = NestedText(expected_type=unicode, allow_none=True)
+    t = NestedText(expected_type=unicode)
     value = Alias("t")
 
     __elements__ = ('rPr', 't')
 
     def __init__(self,
                  rPr=None,
-                 t=None,
+                 t="",
                 ):
         self.rPr = rPr
         self.t = t
@@ -529,12 +529,12 @@ class Paragraph(Serialisable):
     pPr = Typed(expected_type=ParagraphProperties, allow_none=True)
     properties = Alias("pPr")
     endParaRPr = Typed(expected_type=CharacterProperties, allow_none=True)
-    r = Typed(expected_type=RegularTextRun, allow_none=True)
+    r = Sequence(expected_type=RegularTextRun)
     text = Alias('r')
     br = Typed(expected_type=LineBreak, allow_none=True)
     fld = Typed(expected_type=TextField, allow_none=True)
 
-    __elements__ = ('pPr', 'endParaRPr', 'r', 'br', 'fld')
+    __elements__ = ('pPr', 'r', 'br', 'fld', 'endParaRPr')
 
     def __init__(self,
                  pPr=None,
@@ -546,7 +546,7 @@ class Paragraph(Serialisable):
         self.pPr = pPr
         self.endParaRPr = endParaRPr
         if r is None:
-            r = RegularTextRun()
+            r = [RegularTextRun()]
         self.r = r
         self.br = br
         self.fld = fld
